#!/usr/bin/env groovy

import com.urbancode.commons.fileutils.filelister.FileType
import com.urbancode.commons.fileutils.FileUtils
import com.urbancode.vfs.client.Client
import com.urbancode.vfs.client.PatternPathFilter
import com.urbancode.vfs.common.ClientChangeSet
import com.urbancode.vfs.common.ClientChangeSetDelta
import com.urbancode.vfs.common.ClientPathEntry
import com.urbancode.vfs.common.Hash

import org.codehaus.jettison.json.JSONObject;
import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.*;
import java.net.URLEncoder

def workDir = new File('.').canonicalFile
def props = new Properties();
def inputPropsFile = new File(args[0]);
inputPropsFile.withInputStream {
    props.load(it);
}

def repositoryUrl = props['repositoryUrl']
def repositoryId = props['repositoryId']
def label = props['label']
def directoryOffset = props['directoryOffset']
def fileIncludePatterns = props['fileIncludePatterns']
def fileExcludePatterns = props['fileExcludePatterns']

def baseDirectory = new File(directoryOffset)

def includesArray = fileIncludePatterns?.readLines().toArray(new String[0])
def excludesArray = fileExcludePatterns?.readLines().toArray(new String[0])
def excludesList = []
excludesList.addAll(excludesArray)

def proxyHost = System.env['PROXY_HOST'] ? System.env['PROXY_HOST'] : null
def proxyPort = System.env['PROXY_PORT'] ? Integer.valueOf(System.env['PROXY_PORT']) : null

def outputFilePath = props['outputFile']
def changeSetId;

def verbose = false

def syncMode = props['syncMode']
def useSync = syncMode != 'false'
def fullClean = syncMode == 'FULL'
def fullVerification = props['fullVerification'] == 'true'

def syncCleanup = useSync
def resName = props['resName'];
def compName = props['compName'];
def serverURL = props['serverUrl'];
def setFileExecuteBits = (props['setFileExecuteBits'] != null && props['setFileExecuteBits'] == "true");
def verifyFileIntegrity = (props['verifyFileIntegrity'] != null && props['verifyFileIntegrity'] == "true");
def stagedFiles = []
def currentDirs = []

def getLabelForResourceAndComponent = {

    def result = null;

    while (serverURL.endsWith("/")) {
        serverURL = serverURL.substring(0, serverURL.length-1);
    }


    def restUrl = "${serverURL}/rest/inventory/versionByResourceAndComponent/";
    restUrl += URLEncoder.encode(resName)
    restUrl += "/" + URLEncoder.encode(compName)

    def httpClient = new HttpClient();

    if (proxyHost) {
        httpClient.getHostConfiguration().setProxy(proxyHost, proxyPort);
    }
    def methodResult = null;
    def code = null;

    GetMethod method = null;
    try {
        method = new GetMethod(restUrl);
        code = httpClient.executeMethod(method);
        methodResult = method.getResponseBodyAsString();
    }
    finally {
        if (method != null) {
            method.releaseConnection();
        }
    }

    switch (code) {

        case HttpStatus.SC_OK:
            JSONObject responseObject = new JSONObject(methodResult)
            if (!responseObject.isNull("version")) {
                JSONObject versionObject = responseObject.getJSONObject("version")
                if (!versionObject.isNull("name")) {
                    result = versionObject.getString("name")
                }
            }
            break;

        case HttpStatus.SC_NOT_FOUND:
            println "Server returned 404 for getting the previous label!";
            break;

        case HttpStatus.SC_INTERNAL_SERVER_ERROR:
            println "Server error retrieving previous label : " + methodResult;
            break;
        default:
            println "Could not get previous label! Method returned code : " + code;
            break;
    }

    return result;
}




//
// Validation
//

if (baseDirectory.isFile()) {
    throw new IllegalArgumentException("Base directory ${baseDirectory} is a file!")
}

//
// Configure logging
//
new com.urbancode.vfs.client.Logging().configureConsoleLogging()

//
// Download the files
//

def client = new Client(repositoryUrl, proxyHost, proxyPort)
client.setSetFileExecuteBitsOnly(setFileExecuteBits);
client.setVerifyFileIntegrity(verifyFileIntegrity);
//client.setUseCompression(useCompression);
String prevLabel = null;
if (useSync) {
    try {
        prevLabel = getLabelForResourceAndComponent();
    }
    catch (Exception e) {
        println "Could not get label for the previous version! Will not be able to sync!";
        println "${e.message}"
        prevLabel = null;
    }

    if (prevLabel == null) {
        syncCleanup = false;
        println "Could not get label for the previous version.";
    }
}

ClientChangeSet baseChangeSet = null;
def filter = null;

println "Working directory: ${baseDirectory.path}"
println "Including ${fileIncludePatterns}"
println "Excluding ${fileExcludePatterns}"
println ""

if (useSync) {

    filter = new PatternPathFilter(includesArray, excludesArray)

    if (fullClean || syncCleanup && prevLabel != null) {
        println ""
        if (fullClean) {
            println "Removing files that do not exist in the current changeset..."
        } else {
            println "Removing files from the previous changeset which no longer exist in the current one..."
        }

        ClientChangeSet newChangeSet = client.getChangeSetByLabel(repositoryId, label);
        ClientPathEntry[] newEntries = newChangeSet.getEntries();
        def newPaths = newEntries.collect { it.getPath() };

        if (fullClean) { 
            newFiles = newPaths.collect { new File(baseDirectory, it) }

            def rmDirs = []
            //Remove files not in current ChangeSet
            baseDirectory.eachFileRecurse() { curFile ->
                def keepFile = newFiles.find { it.equals(curFile) }

                if (!keepFile && filter.includes(curFile.path)) {
                    if (!curFile.isDirectory()) {
                        println "Deleting '"+curFile.path+"'"
                        curFile.delete()
                    }
                    else {
                        rmDirs << curFile
                    }
                }
            }

            rmDirs.each { directory ->
                if (directory.exists()) {
                    /*
                    println "recursing into "+directory.path
                    directory.eachFileRecurse {
                        if (it.exists()) {
                            println "Deleting '"+it.path+"'"
                            it.delete()
                        }
                    }
                    if (directory.exists()) {
                        println "Deleting '"+it.path+"'"
                        directory.delete()
                    }
                    */
                    new AntBuilder().delete(dir: directory.canonicalPath)
                }
            }

            //Remove files with different type than in ChangeSet
            newEntries.each { newEntry ->
                def curFile = new File(baseDirectory, newEntry.path)
                if (curFile.isDirectory() && !(newEntry.type == FileType.DIRECTORY)) {
                    println "Deleting '"+curFile.path+"'"
                    curFile.delete()
                }
                else if (curFile.isFile() && !(newEntry.type == FileType.REGULAR)) {
                    println "Deleting '"+curFile.path+"'"
                    curFile.delete()
                }
            }
        }
        else {
            ClientChangeSet curChangeSet = client.getChangeSetByLabel(repositoryId, prevLabel);
            ClientPathEntry[] previousEntries = curChangeSet.getEntries();
            def previousPaths = previousEntries.collect { it.path }

            previousPaths.each { curPath ->
                if (!newPaths.contains(curPath) && filter.includes(curPath)) {
                    println "Deleting '"+curPath+"'"
                    File deletable = new File(curPath);
                    if (deletable.exists()) {
                        deletable.delete();
                    }
                }
            }
        }
    }

    baseChangeSet = client.getChangeSetByLabel(repositoryId, label)
    List<ClientPathEntry> entries = Arrays.asList(baseChangeSet.getEntries())
    ClientChangeSetDelta changeSetDelta = new ClientChangeSetDelta(repositoryId, entries)


    def hashedCount = 0
    def unhashedCount = 0
    println ""
    println "Calculating hashes..."
    entries.each {
        if (filter.includes(it.path)) {
            if (it.type.equals(FileType.REGULAR)) {
                def file = new File(it.path)
                if (it.contentHash && it.contentHash.algorithm) {
                    if (file.exists() && !file.isDirectory()) {
                        def hash = null
                        try {
                            boolean matches = false;
                            if (!fullVerification) {
                                matches = it.lastModified == file.lastModified() &&
                                        it.length == file.length()
                            }

                            if (!matches) {
                                hash = Hash.hashForFile(file, it.contentHash.algorithm)
                                hashedCount ++
                            } else {
                                unhashedCount ++
                            }

                            if (matches || hash && hash.equals(it.contentHash)) {
                                changeSetDelta.removeEntry(it)
                                stagedFiles << it.path
                            }
                        }
                        catch (Exception e) {
                            throw new RuntimeException(
                                    "Error calculating hash for ${it.path}: ${e.message}")
                        }
                    }
                }
                else {
                    throw new RuntimeException(
                            "Could not determine hash algorithm for file ${it.path}: ${it.contentHash}")
                }
            }
            else if (it.type.equals(FileType.DIRECTORY)) {
                def file = new File(it.path)
                if (file.isDirectory()) {
                    changeSetDelta.removeEntry(it)
                    currentDirs << it.path
                }
            }
        }
    }

    println "Computed hashes for ${hashedCount} files."

    if (!fullVerification) {
        println "Skipped hashing on ${unhashedCount} files that matched size and date modified."
    }

    def downloadCount = changeSetDelta.entries.length
    def msg = "\nDownloading "+downloadCount+" file"
    msg += downloadCount == 1 ? "..." : "s..."
    println msg
    client.downloadChangeSetDelta(
            changeSetDelta,
            baseDirectory,
            includesArray,
            excludesArray)

    println "Download complete\n"
    stagedFiles.sort()
    stagedFiles.each {
        println "Not changed: './${it}'"
    }
}
else {
    println "Downloading files..."
    client.downloadChangeSetFilesByLabel(
            repositoryId,
            label,
            baseDirectory,
            includesArray,
            excludesArray)

    println "Download complete"
}
