#!/usr/bin/env groovy

import com.urbancode.vfs.client.Client
import com.urbancode.vfs.common.ClientChangeSet
import com.urbancode.vfs.common.ClientPathEntry
import com.urbancode.commons.fileutils.filelister.FileListerBuilder.Directories;
import com.urbancode.commons.fileutils.filelister.FileListerBuilder.Permissions;
import com.urbancode.commons.fileutils.filelister.FileListerBuilder.Symlinks;
import com.urbancode.commons.fileutils.filelister.FileType;
import java.net.URLDecoder;

def workDir = new File('.').canonicalFile
def props = new Properties();
def inputPropsFile = new File(args[0]);
inputPropsFile.withInputStream {
    props.load(it);
}

def repositoryUrl = props['repositoryUrl']
def repositoryId = props['repositoryId']
def changeUser = props['changeUser']
def changeComment = props['changeComment']
def directoryOffset = props['directoryOffset']
def fileIncludePatterns = props['fileIncludePatterns']
def fileExcludePatterns = props['fileExcludePatterns']
def label = props['label'];
def update = props['update'];
def saveFileExecuteBits = props['saveFileExecuteBits'] != null && Boolean.valueOf(props['saveFileExecuteBits']);

def baseDirectory = new File(directoryOffset)

def includesArray = fileIncludePatterns?.readLines().toArray(new String[0])
def excludesArray = fileExcludePatterns?.readLines().toArray(new String[0])

def proxyHost = System.env['PROXY_HOST'] ? System.env['PROXY_HOST'] : null
def proxyPort = System.env['PROXY_PORT'] ? Integer.valueOf(System.env['PROXY_PORT']) : null

def outputFilePath = props['outputFile']

def verbose = false

//
// Validation
//

if (baseDirectory.isFile()) {
    throw new IllegalArgumentException("Base directory ${baseDirectory} is a file!")
}

//
// Collect the files to upload
//

def directories = Directories.INCLUDE_ALL;
def symlinks = Symlinks.AS_LINK;
def hashAlgorithm = "SHA-256";

def permissions = Permissions.NONE;
if (saveFileExecuteBits) {
    permissions = Permissions.FILE_EXECUTE_ONLY;
}

def entries = ClientPathEntry.createPathEntriesFromFileSystem(
        baseDirectory, 
        includesArray, 
        excludesArray, 
        directories, 
        permissions, 
        symlinks, 
        hashAlgorithm) as List;

//
// Upload the files
//

def client = new Client(repositoryUrl, proxyHost, proxyPort)

String stageId = client.createStagingDirectory()
println("Created staging directory: ${stageId}")

try {
    entries.each() { entry ->
        def entryFile = new File(baseDirectory, entry.path)
        println("Adding ${entry.path} to staging directory...")
        client.addFileToStagingDirectory(stageId, entry.path, entryFile)
    }

    ClientChangeSet oldChangeSet = client.getChangeSetByLabel(repositoryId, label);
    if (update && oldChangeSet != null) {
        def oldEntries = oldChangeSet.getEntries();
        oldEntries.each { entry->
            def newEntry = entries.find {it.path == entry.path };
            if (newEntry == null) {
                println("Adding ${entry.path} from old changeset to entries...")
                entries.add(entry);
            }
        }
    }

    ClientChangeSet changeSet = ClientChangeSet.newChangeSet(repositoryId, changeUser, changeComment, entries as ClientPathEntry[])

    println("Committing change set...")
    String changeSetId = client.commitStagingDirectory(stageId, changeSet)
    println("Created change set: ${changeSetId}")

    if (label != null && label.trim() != "") {
        println("Labeling change set with label : ${label}");
        client.labelChangeSet(repositoryId, URLDecoder.decode(changeSetId), label, changeUser, changeComment);
        println ("Done labeling change set!");
    }
}
catch (Exception e) {
    println(e)
    client.deleteStagingDirectory(stageId)
    println("Deleted staging directory: ${stageId}")
    System.exit(1)
}
